﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Collections.Specialized;
using System.ComponentModel;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Windows;
using System.Windows.Data;
using System.Windows.Interactivity;
using Infragistics.Windows.DockManager;
using Infragistics.Windows.DockManager.Events;

namespace WpfApplication4
{
	/// <summary>
	/// Interaction logic for MainWindow.xaml
	/// </summary>
	public partial class MainWindow : Window
	{
		public MainWindow()
		{
			InitializeComponent();
		}


		#region Events

		private void DocumentContentHost_OnActiveDocumentChanged(object sender, RoutedPropertyChangedEventArgs<ContentPane> e)
		{
			ThingsViewModel things = (ThingsViewModel)this.DataContext;

			if (e.NewValue != null)
			{
				things.SelectedThing = (ThingViewModel)e.NewValue.DataContext;
			}
		}


		#endregion

	}

	#region ViewModels

	public class ThingsViewModel : ViewModelBase
	{
		public ThingsViewModel()
		{
			Things = new ObservableCollection<ThingViewModel>();

			Things.Add(new ThingViewModel() {Name = "Tab 1"});
			Things.Add(new ThingViewModel() {Name = "Tab 2"});
			Things.Add(new ThingViewModel() {Name = "Tab 3"});
			Things.Add(new ThingViewModel() {Name = "Tab 4"});
			Things.Add(new ThingViewModel() {Name = "Tab 5"});
		}

		private ObservableCollection<ThingViewModel> _things;
		public ObservableCollection<ThingViewModel> Things { get { return _things; } set { _things = value; this.RaisePropertyChanged();} }

		private ThingViewModel _selectedThing;
		public ThingViewModel SelectedThing { get { return _selectedThing; } set { _selectedThing = value; this.RaisePropertyChanged();} }
	}

	public class ThingViewModel : ViewModelBase
	{
		private string _name;
		public string Name { get { return _name; } set { _name = value; this.RaisePropertyChanged();} }
	}

	/// <summary>
	/// Class to serve as base class for MVVM View Models in WPF
	/// </summary>
	public class ViewModelBase : INotifyPropertyChanged
	{
		/// <summary>
		/// Public event to fire when a property changes
		/// </summary>
		public event PropertyChangedEventHandler PropertyChanged = null;

		/// <summary>
		/// Raises a property changed event
		/// </summary>
		/// <param name="property">The name of the property.  If not specified, it will take the name of the property it is called from</param>
		public void RaisePropertyChanged([CallerMemberName] String property = null)
		{
			this.PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(property));
		}
	}

	#endregion


	#region Behavior
	internal class TabGroupPaneItemsSourceBehavior : Behavior<TabGroupPane>
	{
		#region HeaderMemberPath Property
		public static readonly DependencyProperty HeaderMemberPathProperty = DependencyProperty.Register("HeaderMemberPath", typeof(String), typeof(TabGroupPaneItemsSourceBehavior));
		public String HeaderMemberPath
		{
			get
			{
				return ((String)this.GetValue(HeaderMemberPathProperty));
			}
			set
			{
				this.SetValue(HeaderMemberPathProperty, value);
			}
		}
		#endregion

		#region HeaderTemplate Property
		public static readonly DependencyProperty HeaderTemplateProperty = DependencyProperty.Register("HeaderTemplate", typeof(DataTemplate), typeof(TabGroupPaneItemsSourceBehavior), new PropertyMetadata(null));
		public DataTemplate HeaderTemplate
		{
			get
			{
				return ((DataTemplate)this.GetValue(HeaderTemplateProperty));
			}
			set
			{
				this.SetValue(HeaderTemplateProperty, value);
			}
		}
		#endregion

		#region ItemSource Property
		public static readonly DependencyProperty ItemsSourceProperty = DependencyProperty.Register("ItemsSource", typeof(IList), typeof(TabGroupPaneItemsSourceBehavior), new PropertyMetadata(null, new PropertyChangedCallback(OnItemsSourcePropertyChanged)));
		public IList ItemsSource
		{
			get
			{
				return ((IList)this.GetValue(ItemsSourceProperty));
			}
			set
			{
				this.SetValue(ItemsSourceProperty, value);
			}
		}

		private static void OnItemsSourcePropertyChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
		{
			TabGroupPaneItemsSourceBehavior behavior = d as TabGroupPaneItemsSourceBehavior;
			behavior?.OnItemsSourcePropertyChanged((IList)e.OldValue, (IList)e.NewValue);
		}

		private void OnItemsSourcePropertyChanged(IList oldvalue, IList newvalue)
		{
			base.AssociatedObject.Items.Clear();

			var oldcollection = oldvalue as INotifyCollectionChanged;
			if (oldcollection != null)
				oldcollection.CollectionChanged -= CollectionChanged_CollectionChanged;

			if (newvalue != null)
			{
				var newcollection = newvalue as INotifyCollectionChanged;
				if (newcollection != null)
					newcollection.CollectionChanged += CollectionChanged_CollectionChanged;

				foreach (var item in newvalue)
				{
					ContentPane cp = this.PrepareContainerForItem(item);
					base.AssociatedObject.Items.Add(cp);
				}
			}

			if (newvalue == null || newvalue.Count == 0)
			{
				// clear all panes
				XamDockManager manager = XamDockManager.GetDockManager(base.AssociatedObject);

				if (manager != null)
				{
					List<ContentPane> contentpanes = manager.GetPanes(PaneNavigationOrder.VisibleOrder).ToList();

					foreach (ContentPane cp in contentpanes)
					{
						cp.CloseAction = PaneCloseAction.RemovePane;
						cp.ExecuteCommand(ContentPaneCommands.Close);
					}
				}
			}
		}

		private void CollectionChanged_CollectionChanged(Object sender, NotifyCollectionChangedEventArgs e)
		{
			if (e.Action == NotifyCollectionChangedAction.Remove)
			{
				XamDockManager manager = XamDockManager.GetDockManager(base.AssociatedObject);
				if (manager != null)
				{
					IEnumerable<ContentPane> contentpanes = manager.GetPanes(PaneNavigationOrder.VisibleOrder);
					foreach (ContentPane cp in contentpanes)
					{
						var dc = cp.DataContext;
						if (dc != null && e.OldItems.Contains(dc))
						{
							cp.ExecuteCommand(ContentPaneCommands.Close);
						}
					}
				}
			}
			else if (e.Action == NotifyCollectionChangedAction.Add)
			{
				foreach (var item in e.NewItems)
				{
					ContentPane cp = this.PrepareContainerForItem(item);
					if (cp != null)
					{
						base.AssociatedObject.Items.Add(cp);
						cp.Activate();
					}
				}
			}
		}

		protected ContentPane PrepareContainerForItem(Object item)
		{
			ContentPane cp = new ContentPane();

			cp.Content = item;
			cp.DataContext = item;

			if (this.HeaderTemplate != null)
				cp.HeaderTemplate = this.HeaderTemplate;

			if (this.ItemTemplate != null)
				cp.ContentTemplate = this.ItemTemplate;

			cp.CloseAction = PaneCloseAction.RemovePane;
			cp.Closing += this.Container_Closing;

			this.CreateBindings(item, cp);

			return (cp);
		}

		private void Container_Closing(Object sender, PaneClosingEventArgs e)
		{
			ContentPane cp = sender as ContentPane;
			if (cp != null)
			{
				var item = cp.DataContext;
				if (this.ItemsSource != null && this.ItemsSource.Contains(item))
				{
					ThingViewModel thing = (ThingViewModel)item;

					if (MessageBox.Show($"Are you sure you want to delete this thing '{thing.Name}'?", "Delete thing", MessageBoxButton.YesNo, MessageBoxImage.Question) == MessageBoxResult.Yes)
					{
						cp.Closing -= this.Container_Closing; //no memory leaks
						this.ItemsSource.Remove(item);
						this.RemoveBindings(cp);
					}
					else
					{
						e.Cancel = true;
					}
				}
			}
		}

		private void CreateBindings(Object item, ContentPane cp)
		{
			if (!String.IsNullOrWhiteSpace(this.HeaderMemberPath))
			{
				Binding binding = new Binding(this.HeaderMemberPath);
				binding.Source = item;
				cp.SetBinding(ContentPane.HeaderProperty, binding);
				cp.AllowFloatingOnly = false;
				cp.AllowDocking = false;
			}
		}

		private void RemoveBindings(ContentPane cp)
		{
			cp.ClearValue(ContentPane.HeaderProperty);
		}

		#endregion

		#region ItemTemplate Property
		public static readonly DependencyProperty ItemTemplateProperty = DependencyProperty.Register("ItemTemplate", typeof(DataTemplate), typeof(TabGroupPaneItemsSourceBehavior), new PropertyMetadata(null));
		public DataTemplate ItemTemplate
		{
			get
			{
				return ((DataTemplate)this.GetValue(ItemTemplateProperty));
			}
			set
			{
				this.SetValue(ItemTemplateProperty, value);
			}
		}
		#endregion
	}

	#endregion
}
